'''
Based on https://www.tensorflow.org/beta/guide/keras/saving_and_serializing

    Inference using a custom model created using tensorflow 2.0.
    This example is using the MNIST database of handwritten digits
    (http://yann.lecun.com/exdb/mnist/)
    Author: tensorflow
    Project: https://github.com/tensorflow/docs
    Modifications:

*****************************************************************
Licensed Materials - Property of IBM
(C) Copyright IBM Corp. 2019. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*****************************************************************
'''

import mnist_custom_model
import argparse

import tensorflow as tf
import numpy as np

def main ():
    (x_train, y_train), (x_test, y_test) = tf.keras.datasets.mnist.load_data()
    x_train = x_train.reshape(60000, 784).astype('float32') / 255
    x_test  = x_test.reshape(10000, 784).astype('float32') / 255

    # Recreate the model
    new_model = mnist_custom_model.get_model()
    new_model.compile(loss='sparse_categorical_crossentropy',
                      optimizer=tf.keras.optimizers.RMSprop(),
                      metrics=['accuracy'])

    # This initializes the variables used by the optimizers,
    # as well as any stateful metric variables
    new_model.train_on_batch(x_train[:1], y_train[:1])

    # Load the state of the old model
    new_model.load_weights(FLAGS.saved_model_path)

    # Check that the model state has been preserved
    new_preds = new_model.predict(x_test)
    new_preds = np.array(tf.argmax(new_preds, 1))
    print ("Prediction Accuracy:", np.sum(new_preds == y_test) / y_test.shape[0])

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--saved_model_path', type=str, default="/tmp/mnist-model/mnist-custom-training-weights")
    FLAGS, unparsed = parser.parse_known_args()

    main()
