'''
Based on https://www.tensorflow.org/beta/guide/keras/saving_and_serializing

    Training a sequential or functional model created using tensorflow 2.0.
    This example is using the MNIST database of handwritten digits
    (http://yann.lecun.com/exdb/mnist/)
    Author: tensorflow
    Project: https://github.com/tensorflow/docs
    Modifications:

*****************************************************************
Licensed Materials - Property of IBM
(C) Copyright IBM Corp. 2019. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*****************************************************************
'''

from __future__ import absolute_import, division, print_function, unicode_literals
import ddl
import argparse
import time
import os

import tensorflow as tf
import numpy as np

from tensorflow import keras
from tensorflow.keras import layers

def main ():
    #Load dataset
    (x_train, y_train), (x_test, y_test) = keras.datasets.mnist.load_data()
    x_train = x_train.reshape(60000, 784).astype('float32') / 255
    x_test = x_test.reshape(10000, 784).astype('float32') / 255

    #Split dataset for DDL purposes
    x_train = np.array_split(x_train, ddl.size())[ddl.rank()]
    y_train = np.array_split(y_train, ddl.size())[ddl.rank()]

    #Define a simple model
    inputs  = keras.Input(shape=(784,), name='digits')
    x       = layers.Dense(64, activation='relu', name='dense_1')(inputs)
    x       = layers.Dense(64, activation='relu', name='dense_2')(x)
    outputs = layers.Dense(10, activation='softmax', name='predictions')(x)

    model   = keras.Model(inputs=inputs, outputs=outputs, name='3_layer_mlp')

    #Compile and train the model (fit)
    model.summary()
    model.compile(loss='sparse_categorical_crossentropy',
                  optimizer=keras.optimizers.RMSprop(),
                  metrics=['accuracy'])
    history = model.fit(x_train, y_train,
                        batch_size=FLAGS.batch_size,
                        epochs=FLAGS.epochs)

    #Save the trained model
    if ddl.rank() == 0:
        print ("Testing Accuracy:", model.evaluate(x_test, y_test)[-1])
        model_dir = FLAGS.saved_model_path + "/"
        model_file = "mnist-seq-training.h5"
        os.makedirs(model_dir, exist_ok=True)
        print ("Saving trained model (using model.save) to ", model_dir + model_file)
        model.save(model_dir + model_file)

if __name__ == '__main__':
    parser = argparse.ArgumentParser()
    parser.add_argument('--epochs', type=int, default=2)
    parser.add_argument('--batch_size', type=int, default=64)
    parser.add_argument('--saved_model_path', type=str, default="/tmp/mnist-model")

    FLAGS, unparsed = parser.parse_known_args()

    main()
