
'''
Based on https://www.tensorflow.org/beta/tutorials/quickstart/beginner

    A simple example of training on MNIST dataset using tensorflow 2.0
    This example is using the MNIST database of handwritten digits
    (http://yann.lecun.com/exdb/mnist/)
    Author: tensorflow
    Project: https://github.com/tensorflow/docs
    Modifications:

*****************************************************************
Licensed Materials - Property of IBM
(C) Copyright IBM Corp. 2019. All Rights Reserved.
US Government Users Restricted Rights - Use, duplication or
disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
*****************************************************************
'''

from __future__ import absolute_import, division, print_function, unicode_literals

from os import environ
import argparse
import ddl
import tensorflow as tf
import numpy as np

print ("TF executing eagerly? ", tf.executing_eagerly())

def main(num_iters):
    #Load and prepare mnist dataset
    mnist = tf.keras.datasets.mnist

    (x_train, y_train), (x_test, y_test) = mnist.load_data()
    x_train, x_test = x_train / 255.0, x_test / 255.0

    x_train = np.array_split(x_train, ddl.size())[ddl.rank()]
    y_train = np.array_split(y_train, ddl.size())[ddl.rank()]

    print ("x_train={}, y_train={}".format(x_train.shape, y_train.shape))

    model = tf.keras.models.Sequential([
                    tf.keras.layers.Flatten(input_shape=(28, 28)),
                    tf.keras.layers.Dense(128, activation='relu'),
                    tf.keras.layers.Dropout(0.2),
                    tf.keras.layers.Dense(10, activation='softmax')
                ])

    model.compile(optimizer='adam',
                  loss='sparse_categorical_crossentropy',
                  metrics=['accuracy'])

    # DDL: Add the DDL callback.
    callbacks = list()
    callbacks.append(ddl.DDLCallback())
    callbacks.append(ddl.DDLGlobalVariablesCallback())

    model.fit(x_train, y_train, epochs=num_iters, callbacks=callbacks)

    if ddl.rank() == 0:
        print ("Testing Accuracy:", model.evaluate(x_test, y_test)[-1])

if __name__ == '__main__':
    parser = argparse.ArgumentParser(
        description='DDL allreduce op testcase')
    parser.add_argument('--ddl_options', type=str,
                        default='-mode b:2x1 -dump_iter 100',
                        help='DDL options. e.g. "-mode b:2x1 -dump_iter 100"')
    parser.add_argument('--num_iterations', type=int,
                        default=5,
                        help='Number of iterations to perform')

    FLAGS, unparsed = parser.parse_known_args()

    if environ.get('DDL_OPTIONS') is None:
        #Need to explicitly call ddl.init
        ddl.init(FLAGS.ddl_options)

    num_iters   = FLAGS.num_iterations
    main(num_iters)
